function behandlegui
% behandlegui Lets you load an image, then
% choose a section and calcualte the contrast
% in the section. It also allows removal of
% background noise by loading the image that
% contains this information. It has
% a function to save the section and a figure
% that contains the contrast and the size
% of the chosen section .
 
   %  Create and then hide the GUI
   %  as it is being constructed.
   f = figure('Visible','off','Position',...
           [360,500,800,400]);
 
   %  Construct the components.

   openi = uicontrol('Style','pushbutton',...
          'String','Open image',...
          'Position',[650,360,80,25],...
          'Callback',{@openi_Callback});
   openbn = uicontrol('Style','pushbutton',...
          'String','Backgr noise',...
          'Position',[650,330,80,25],...
          'Callback',{@openbn_Callback});
   frame = uicontrol('Style','pushbutton',...
          'String','Draw frame',...
          'Position',[650,300,80,25],...
          'Callback',{@frame_Callback});
   savef = uicontrol('Style','pushbutton',...
          'String','Save figure',...
          'Position',[650,270,80,25],...
          'Callback',{@savef_Callback});
   backgr = uicontrol('Style','radio',...
          'String','Backgr noise',...
          'Value',0,...
          'Position',[650,240,90,25],...
          'Callback',{@bn_Callback});
   htext = uicontrol('Style','text','String',...
          'Choose section',...
          'Position',[650,200,120,15]);
   xtext = uicontrol('Style','text',...
          'String','x',...
          'Position',[760,110,30,15]);
   dxtext = uicontrol('Style','text',...
          'String','dx',...
          'Position',[760,80,30,15]);
   ytext = uicontrol('Style','text',...
          'String','y',...
          'Position',[760,170,30,15]);
   dytext = uicontrol('Style','text',...
          'String','dy',...
          'Position',[760,140,30,15]);
   speckletext = uicontrol('Style','text',...
          'String','Speckle contrast:',...
          'Position',[650,40,120,15]);
   speckleval = uicontrol('Style','text',...
          'String','-',...
          'Position',[650,20,60,15]);
   yedit = uicontrol('Style','edit',...
          'String',{'1'},...
          'Position',[650,170,100,25],...
          'Callback',{@edit_Callback});
   dyedit = uicontrol('Style','edit',...
          'String',{'480'},...
          'Position',[650,140,100,25],...
          'Callback',{@edit_Callback});
   xedit = uicontrol('Style','edit',...
          'String',{'1'},...
          'Position',[650,110,100,25],...
          'Callback',{@edit_Callback});
   dxedit = uicontrol('Style','edit',...
          'String',{'640'},...
          'Position',[650,80,100,25],...
          'Callback',{@edit_Callback});

   ha = axes('Units','Pixels','Position',...
          [50,60,550,300]); 
   align([xedit,xtext],'None','Middle')
   align([dxedit,dxtext],'None','Middle')
   align([yedit,ytext],'None','Middle')
   align([dyedit,dytext],'None','Middle')
   

   % Initialize the GUI.
   % Change units to normalized so components resize 
   % automatically.
   set([f,ha,frame,openi,savef,...
       htext,xedit,dxedit,yedit,dyedit],...
       'Units','normalized');

   %Defining som variables to be used.
   pathname = ...
   'D:\HICFODS\Knut Oyvind\bilder master\myfile.bmp';
   scrsz = get(0,'ScreenSize');
   I = [];      % Original image
   I2 = [];     % Copy of original image
                % to be manipulated.
   Id = [];     % Double version of the 
                % image. Used to calculate
                % the speckle contrast.
   
   Ibn = zeros(480,640);    % Double version of
                            % the background noise image
   bn = 0;      % Boolean value that indicates if the
                % user has loaded a back ground image
   
   x = 1;       % Coordinates for the choosen section
   dx = 640;
   y = 1;
   dy = 480;
   
   C = 1;       % The speckle contrast
   
   
   % Assign the GUI a name to appear in the window title.
   set(f,'Name','Speckle contrast in a section')
   % Move the GUI to the center of the screen.
   movegui(f,'center')
   % Make the GUI visible.
   set(f,'Visible','on');
 
   %  Callbacks for behandlegui. These callbacks automatically
   %  have access to component handles and initialized data 
   %  because they are nested at a lower level.
 
      %  frame push button callback. Checks if
      %  the defined area is within the limits
      %  of the image. If it is, it draws the frames
      %  of the area and calcualtes the speckle
      %  contrast.
      function frame_Callback(source,eventdata)
          %Reading the values
          x = str2double(get(xedit,'String'));
          dx = str2double(get(dxedit,'String'));
          y = str2double(get(yedit,'String'));
          dy = str2double(get(dyedit,'String'));
          %Checking the limits
          if((x>0) && (x<641) && ((x+dx)>0) && ((x+dx)<642) ...
              && (y>0) && (y<481) && ((y+dy)>0) && ((y+dy)<482))
              % Eliminates background noise
              Itemp = Id - Ibn;
              % Extracts the defined area
              samp = Itemp(y:(y+dy-1),x:(x+dx-1),:);
              % Calculates the speckle contrast in this area
              C = speckle_contrast(samp);
              % Displays the value in the GUI
              set(speckleval,'String',num2str(C(1)))
              % I is unaltered, we can do whatever we want to I2
              I2 = I;
              % Draws the top horizontal line
              I2(y:(y+1),x:(x+dx-1),1) = 255;
              % Draws the bottom horizontal line
              I2((y+dy-2):(y+dy-1),x:(x+dx-1),1) = 255;
              % Draws the left vertical line
              I2(y:(y+dy-1),x:(x+1),1) = 255;
              % Draws the right vertical line
              I2(y:(y+dy-1),(x+dx-2):(x+dx-1),1) = 255;
              % Shows the image in the GUI
              image(I2);
              xlabel('x');
              ylabel('y');
              axis image
              title(['x: ',num2str(x),', dx: ',num2str(dx),...
                    ', y: ',num2str(y),', dy: ',num2str(dy)]);
          else
              r = errordlg('Index out of range', 'Index error');
              uiwait(r);
          end
        
      end
  
   % Text edit callbacks. It has no function.
   function edit_Callback(source,eventdata) 
   end

   % Radio button callback. It impossible
   % for the user to change the value
   % without opening an image, which should
   % contain the backgrouns noise.
   function bn_Callback(source,eventdata)
       set(backgr,'Value',bn);
   end

   % Background image push button callback. Lets the 
   % user choose an image to open.
   function openbn_Callback(source,eventdata) 
    [filename, pathname] = uigetfile({'*.bmp',...
          'Bitmap (*.bmp)';'*.jpg;*.tif;*.png;*.gif',...
          'All Image Files';'*.*','All Files' },...
          'Choose backgruond noise image',...
          pathname);
      
    if isequal(filename,0)
        disp('User selected Cancel')
    else
        disp(['User selected', fullfile(pathname, filename)])
        Ibn = imread(strcat(pathname, filename));
        Ibn = double(Ibn(:,:,1))/255; % Allows negative values
        bn = 1;
        set(backgr,'Value',1);
    end
   end

   % Open image push button callbacks. Lets the user choose 
   % an image to open.
   function openi_Callback(source,eventdata) 
    [filename, pathname] = uigetfile({'*.bmp','Bitmap (*.bmp)';...
          '*.jpg;*.tif;*.png;*.gif','All Image Files';...
          '*.*','All Files' },'Velg bilde til  behandle',...
          pathname);
      
    if isequal(filename,0)
        disp('User selected Cancel')
    else
        disp(['User selected', fullfile(pathname, filename)])
        I = imread(strcat(pathname, filename));
        Id = double(I(:,:,1))/255;
    end
    % Shows the image in the GUI.
    image(I);
    xlabel('x');
    ylabel('y');
    axis image
   end
 
   % Save figure push button callbacks. It makes
   % a new figure and saves it to a user
   % spesified loaction together with two images,
   % the image with the frame and the section
   % in the frame.
   function savef_Callback(source,eventdata)    
   h = figure('Position',...
            [10 scrsz(4)/3 (scrsz(3) - 20) scrsz(4)/2])
   % Plots the original image with the frame.
   subplot(1,2,1);
   image(I2)
   xlabel('x');
   ylabel('y');
   title(['x: ',num2str(x),', dx: ',num2str(dx),...
         ', y: ',num2str(y),', dy: ',num2str(dy)]);
   axis image;
   
   Itemp = Id - Ibn;    % Removes background noise
   % Picks out the chosen area.
   samp = Itemp(y:(y+dy-1),x:(x+dx-1),:);
   % Calculates the speckle contrast.
   C = speckle_contrast(samp);
   % Picks out the section from the image to be plotted.
   dI = I(y:(y+dy-1),x:(x+dx-1),:);
   % Plots the section.
   subplot(1,2,2);
   image(dI);
   xlabel('x');
   ylabel('y');
   title(['Speckle contrast ',num2str(C(1))]);
   axis image;
   % Saves the selected section, the original image
   % with the frame.
   lagreBilde2(samp,I2,h);
   end 
 
end 