% speckle_size.m is a script that estimates
% the speckle size. It uses the power spectral
% density (PSD) to do this.


% Loads the speckle image
I = imread('path to image');

% The average spatial sampling frequency of the pixels
fs = 1/(9.635e-6); 

%Finding the autocorrelation
% Choosing one of the layers in the image
b = double(I(:,:,1))./255;   

% m and n are the numbers of poinst in
% the fast Fourier analysis
m = 2560;
n = 1920;

% Finding the Fourier transform
B = fft2(b,n,m);
% Finding abs^2, i.e the PSD
PSD = B.*conj(B); 
% Sorting the result after using fft
PSD2 = fftshift(PSD);       


% Singels out the center line along x
y = PSD((n/2+1),(m/2+1):m);
% The frequency axis 
f = fs/m*(0:(m/2));         

% We expect a linear PSD. To estimate the point
% where the PSD reach zero, we singel out the points
% that follow a linear trend, i.e. the first pionts 
% as long as the slope is negative.
run = 1;
i = 1;
while run
    if y(i)<y(i+1)
        run = 0;
        disp(i);
    else
        i = i + 1;
    end
end


% We do a very simple regression based on
% these points.
% Slope:
a = (y(i)-y(1))/(i-1);
% The point where it crosses the x-axis,
% i.e. y=0, is the max frequency.
fm = -y(1)/a;
s = fm*fs/m;
% The invers of the max frequency gives
% the smalles speckle size.
xwidth = (1/s)*1e+3;
% The data set needed to plot the line
y2 = [y(1) 0];
f2 = [0 s];
%Plotting the PSD and the line
figure;
hold all
plot(f(1:15),y(1:15),'LineWidth',2);
plot(f2,y2,'--','LineWidth',2)

title(['Center line (x), \sigma_s: ',num2str(xwidth),' mm'],...
    'FontSize',13);
legend('PSD','Approximation')
ylabel('Power Spectral Density','FontSize',13);
xlabel('Spatial frequency','FontSize',13);

% We repete the calculations along y
% Singels out the center line along y
y = PSD((n/2+1):n,(m/2+1));
% The frequency axis 
f = fs/n*(0:n); 
run = 1;
i = 1;
while run
    if y(i)<y(i+1)
        run = 0;
        disp(i);
    else
        i = i + 1;
    end
end
a = (y(i)-y(1))/(i-1);
fm = -y(1)/a;
s = fm*fs/n;
ywidth = (1/s)*1e3;
y2 = [y(1) 0];
f2 = [0 s];
figure;
hold all
plot(f(1:15),y(1:15),'LineWidth',2);
plot(f2,y2,'--','LineWidth',2)

title(['Center line (y), \sigma_s: ',num2str(ywidth),' mm'],...
    'FontSize',13);
legend('PSD','Approximation')
ylabel('Power Spectral Density','FontSize',13);
xlabel('Spatial frequency','FontSize',13);